//
// This file is part of the book "Language C++ et calcul scientifique"
//
// Copyright (C) 2005-2012 Pierre Saramito <Pierre.Saramito@imag.fr>
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
#include "mesh_vtk.h"
#include "valarray_io.h"
template <typename T, size_t D>
void fem_vtk (std::ostream& vtk,
    const mesh<T,D>& Th, const std::valarray<T>& uh) {
  using namespace std;
  mesh_vtk (vtk, Th);
  vtk << "POINT_DATA " << uh.size() << endl
      << "SCALARS fem float" << endl
      << "LOOKUP_TABLE default"  << endl;
  for (size_t i = 0; i < uh.size(); ++i) 
    vtk << uh[i] << endl;
}
