//
// This file is part of the book "Language C++ et calcul scientifique"
//
// Copyright (C) 2005-2012 Pierre Saramito <Pierre.Saramito@imag.fr>
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
#include <iostream>
using namespace std;
template <class T>
T pow_recursive (const T& x, size_t n) {
  if (n == 0) return 1;
  if (n == 1) return x;
  T y = pow_recursive(x,n/2);
  return (n % 2 == 0) ? y*y : y*y*x;
}
int main(int argc, char** argv) {
  double x = (argc > 1) ? atof(argv[1]) : 2;
  size_t n = (argc > 2) ? atoi(argv[2]) : 10;
  cout << pow_recursive(x,n) << endl;
}
