//
// This file is part of the book "Language C++ et calcul scientifique"
//
// Copyright (C) 2005-2012 Pierre Saramito <Pierre.Saramito@imag.fr>
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
#include "mesh.h"
#include <valarray>
template <typename T>
void fem_gdat (std::ostream& gdat,
  const mesh<T,2>& Th, const std::valarray<T>& uh) {
  using namespace std;
  for (size_t e = 0; e < Th.size(); e++) {
    size_t i = Th[e][0], j = Th[e][1], k = Th[e][2];
    const point<T,2>& a = Th.vertex(i),
                      b = Th.vertex(j),
                      c = Th.vertex(k);
    gdat << a[0] << " " << a[1] << " " << uh[i] << "\n"
	 << b[0] << " " << b[1] << " " << uh[j] << "\n\n"
	 << c[0] << " " << c[1] << " " << uh[k] << "\n"
	 << c[0] << " " << c[1] << " " << uh[k] << "\n\n\n";
  }
}
