BEGIN { state = 0; }
($1 == "") { next; }
(state == 0 && $1 != "Vertices") { next; }
(state == 0 && $1 == "Vertices") { state = 1; next; }
(state == 1) { print; nv = $1; iv = 0; state = 2; next; }
(state == 2) { print; ++iv; if (iv == nv) state = 3; next; }
(state == 3 && $1 == "Triangles") { state = 4; next; }
(state == 4) { print; ne = $1; ie = 0; state = 5; next; }
(state == 5) { print; ++ie; if (ie == ne) state = 6; next; }
