//
// This file is part of the book "Language C++ et calcul scientifique"
//
// Copyright (C) 2005-2012 Pierre Saramito <Pierre.Saramito@imag.fr>
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
#include "mesh.h"
template <typename T>
void mesh_vtk (std::ostream& vtk, const mesh<T,3>& Th) {
  using namespace std;
  vtk << "# vtk DataFile Version 1.0" << endl
      << "Unstructured Grid" << endl
      << "ASCII" << endl
      << "DATASET UNSTRUCTURED_GRID" << endl
      << "POINTS " << Th.n_vertex() << " float" << endl;
  for (size_t i = 0; i < Th.n_vertex(); i++) 
    vtk << Th.vertex(i) << endl;
  vtk << "CELLS " << Th.size() << " " << 5*Th.size() << endl;
  for (size_t e = 0; e < Th.size(); e++) {
    vtk << "4";
    for (size_t ie = 0; ie < 4; ie++)
      vtk << " " << Th[e][ie];
    vtk << endl;
  }
  vtk << "CELL_TYPES " << Th.size() << endl;
  for (size_t e = 0; e < Th.size(); e++)
    vtk << "10" << endl;
}
