//
// This file is part of the book "Language C++ et calcul scientifique"
//
// Copyright (C) 2005-2012 Pierre Saramito <Pierre.Saramito@imag.fr>
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
template<typename T, size_t D>
void renumbering(const mesh<T,D>& Th, std::valarray<size_t>& num){
  num.resize (Th.n_vertex());
  size_t i_bdr = 0, i_int = 0;
  for (size_t i = 0; i < Th.n_vertex(); i++)
    if (Th.is_boundary (i))
      num[i] = i_bdr++;
    else
      num[i] = i_int++;
}
