//
// This file is part of the book "Language C++ et calcul scientifique"
//
// Copyright (C) 2005-2012 Pierre Saramito <Pierre.Saramito@imag.fr>
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
template <typename T>
void energy (const std::valarray<T>& x, std::valarray<T>& D,
  std::valarray<T>& L)
{
  for (size_t i = 0; i < x.size()-2; i++)
    D [i] = 1/(x[i+1]-x[i]) + 1/(x[i+2]-x[i+1]);
  for (size_t i = 0; i < x.size()-3; i++) 
    L [i] = - 1/(x[i+2]-x[i+1]);
  L [x.size()-3] = 0;
}
