//
// This file is part of the book "Language C++ et calcul scientifique"
//
// Copyright (C) 2005-2012 Pierre Saramito <Pierre.Saramito@imag.fr>
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
template <typename T, size_t D>
void assembly_mass (const mesh<T,D>& Th,
    const std::valarray<size_t>& num, std::valarray<T>& M) { 
  M.resize (Th.n_internal_vertex());
  M = 0;
  for (size_t e = 0; e < Th.size(); e++) {
    T c = meas(Th,e)/(D+1);
    for (size_t r = 0; r < D+1; r++) {
      size_t i = Th[e][r];
      if (! Th.is_boundary(i)) M[num[i]] += c;
    }
  }
}
