//
// This file is part of the book "Language C++ et calcul scientifique"
//
// Copyright (C) 2005-2012 Pierre Saramito <Pierre.Saramito@imag.fr>
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
#include "mesh.h"
#include "matrix.h"
#include "renumbering.h"
#include "assembly_mass.h"
#include "assembly_energy.h"
#include "interpolate.h"
#include "valarray_util.h"
#include "cg.h"
#include "eye.h"
template <typename T, size_t D>
std::valarray<T>
dirichlet (const mesh<T,D>& Th, const std::valarray<T>& fh) {
  std::valarray<size_t> num;
  std::valarray<T>      M;
  matrix<T>             A;
  renumbering     (Th, num);
  assembly_mass   (Th, num, M);
  assembly_energy (Th, num, A);
  std::valarray<bool> internal (Th.n_vertex());
  for (size_t i = 0; i < Th.n_vertex(); i++)
    internal[i] = ! Th.is_boundary(i);
  std::valarray<T> b = M*fh[internal];
  std::valarray<T> x (0., b.size());
  cg (A, x, b, eye(), 100000, 1e-10);
  std::valarray<T> uh (0.0, Th.n_vertex());
  uh [internal] = x;
  return uh;
}
