//
// This file is part of the book "Language C++ et calcul scientifique"
//
// Copyright (C) 2005-2012 Pierre Saramito <Pierre.Saramito@imag.fr>
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
#include "valarray_io.h"
#include "index.h"
using namespace std;
int main (int argc, char** argv) {
  valarray<double> u; 
  cin >> u;
  size_t m = sqrt(u.size()) - 1;
  index_type elt(m+1);
  double h = 2./m;
  for (size_t i = 0; i < m+1; i++) {
    for (size_t j = 0; j < m+1; j++)
      cout << -1+i*h <<" "<< -1+j*h <<" "<< u[elt(i,j)] << endl;
    cout << endl;
  }
}
