//
// This file is part of the book "Language C++ et calcul scientifique"
//
// Copyright (C) 2005-2012 Pierre Saramito <Pierre.Saramito@imag.fr>
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
#include <valarray>
struct row_type {
  row_type (size_t n1) : n(n1) {}
  std::slice operator() (size_t j) const {
	return std::slice (n*j, n, 1); }
protected:
  size_t n;
};
struct col_type {
  col_type (size_t n1) : n(n1) {}
  std::slice operator() (size_t i) const {
	return std::slice (i, n, n); }
protected:
  size_t n;
};
